from datetime import datetime
from typing import Union
from uuid import UUID

from dateutil import parser


class PaymentResponse:
    payment_id: str
    seller_id: UUID
    amount: int
    currency: str
    order_id: str
    status: str
    received_at: str

    def __init__(
        self,
        payment_id: str,
        seller_id: Union[UUID, str],
        amount: int,
        currency: str,
        order_id: str,
        status: str,
        received_at: str = None,
    ):
        self.payment_id = payment_id
        self.seller_id = seller_id if isinstance(seller_id, UUID) else UUID(seller_id)
        self.amount = amount
        self.currency = currency
        self.order_id = order_id
        self.status = status
        self.received_at = (
            received_at
            if isinstance(received_at, datetime) or received_at is None
            else parser.isoparse(received_at)
        )



from typing import Optional
from pydantic import BaseModel, ValidationError, validator


class AuthenticationRequest(BaseModel):
  client_id: str
  client_secret: str


class AuthenticationResponse(BaseModel):
    access_token: Optional[str]
    expires_in: Optional[int]



from typing import Optional, List
from pydantic import BaseModel, ValidationError, validator


class AddressSchema(BaseModel):
  street: str
  number: str
  complement: str
  district: str
  city: str
  state: str
  country: str
  postal_code: str


class CustomerSchema(BaseModel):
  seller_id: str
  customer_id: str
  first_name: str
  last_name: str
  document_type: str
  document_number: str
  birth_date: str
  phone_number: str
  celphone_number: str
  email: str
  observation: str
  status: str
  address: AddressSchema




import unittest

import pytest

from getnet.services.token import CardNumber


def test_invalid_card_number():
    with pytest.raises(AttributeError):
        CardNumber("123", "123")


def test_invalid_customer_id():
    with pytest.raises(AttributeError):
        CardNumber("5155901222280001", "a" * 101)


def test_get_as_dict():
    object = CardNumber("5155901222280001", "customer_21081826")
    assert {
        "card_number": "5155901222280001",
        "customer_id": "customer_21081826",
    } == object.as_dict()



"""
Module implements the Card service responses
"""

import uuid
from datetime import datetime
from typing import Union

from dateutil import parser

from getnet.domain.cards.card import Card
from getnet.domain.token.card_token import CardToken


class NewCardResponse(object):
    """Represents the new card registration on safe response"""

    card_id: uuid.UUID
    number_token: CardToken

    def __init__(self, card_id: str, number_token: Union[CardToken, str]):
        self.card_id = card_id if isinstance(card_id, uuid.UUID) else uuid.UUID(card_id)
        self.number_token = (
            number_token
            if isinstance(number_token, CardToken)
            else CardToken(number_token)
        )


class CardResponse(Card):
    """Represents the card (in safe) responses"""
    card_id: uuid.UUID
    last_four_digits: str
    used_at: datetime
    created_at: datetime
    updated_at: datetime
    status: str
    bin: str

    def __init__(
        self,
        card_id: Union[uuid.UUID, str],
        last_four_digits: str = None,
        used_at: str = None,
        created_at: str = None,
        updated_at: str = None,
        status: str = None,
        bin: str = None,
        **kwargs,
    ):
        self.card_id = card_id if isinstance(card_id, uuid.UUID) else uuid.UUID(card_id)
        self.last_four_digits = last_four_digits
        self.used_at = parser.isoparse(used_at) if used_at else None
        self.created_at = parser.isoparse(created_at) if created_at else None
        self.updated_at = parser.isoparse(updated_at) if updated_at else None
        self.status = status
        self.bin = bin

        kwargs.update({"cardholder_identification": None, "security_code": None})

        super(CardResponse, self).__init__(**kwargs)

    def _as_dict(self):
        data = super(CardResponse, self)._as_dict()
        data.pop("used_at")
        data.pop("created_at")
        data.pop("updated_at")
        return data



"""Implement Token Service"""
from pydantic import ValidationError
from getnet.domain.services import Service as BaseService
from getnet.infra.dtos.card_bin import CardBinResponse


class CardBinInfo(BaseService):
    """Represents the token service operations"""

    path = "/v1/cards/binlookup/"

    def binlookup(self, card_bin: str):
        self._client.request.headers = (
            {
                "Authorization": "Bearer {}".format(self._client.access_token),
            }
        )

        url = f"{self.path}{card_bin}"

        try:
            card_data = CardBinResponse(**self._get(url))
        except ValidationError as e:
            raise e.errors()

        return card_data



"""Implements the Card Token entity"""


class CardVerified:
    """Represent the card number tokenized version"""

    status: bool

    def __init__(self, status: str):
        """
        Args:
            number_token (str):
        """
        self.status = True if status == "VERIFIED" else False



from typing import Optional
from pydantic import BaseModel

class CardVerificationResponse(BaseModel):
    status: str
    verification_id: str
    authorization_code: str
    transaction_id: Optional[str]



from requests.exceptions import RequestException


class RequestError(RequestException):
    def __init__(self, *args, **kwargs):
        self.error_code = kwargs.pop("error_code")
        self.description = kwargs.pop("description")
        super(RequestError, self).__init__(*args, **kwargs)



import os

import pytest
from pytest_mock import MockFixture

from getnet import Client, Environment


@pytest.fixture
def client_mock(mocker: MockFixture):
    return mocker.patch("getnet.Client")


@pytest.fixture(scope="module")
def client():
    return Client(
        os.environ.get("GETNET_SELLER_ID"),
        os.environ.get("GETNET_CLIENT_ID"),
        os.environ.get("GETNET_CLIENT_SECRET"),
        Environment.HOMOLOG,
    )



from ipaddress import IPv4Address
from typing import Union


class Device:
    ip_address: IPv4Address
    device_id: str

    def __init__(self, ip_address: Union[IPv4Address, str], device_id: str):
        if len(device_id) > 80:
            raise TypeError("Device ID too long (max: 80 characters)")

        self.ip_address = (
            ip_address
            if isinstance(ip_address, IPv4Address)
            else IPv4Address(ip_address)
        )
        self.device_id = device_id

    def as_dict(self):
        data = self.__dict__.copy()
        data["ip_address"] = str(self.ip_address)
        return data



from typing import Union

from getnet.domain.customers.address import Address


class Customer:
    seller_id: str
    customer_id: str
    first_name: str
    last_name: str
    document_type: str
    document_number: str
    birth_date: str
    phone_number: str
    celphone_number: str
    email: str
    observation: str
    address: Address

    def __init__(
        self,
        first_name: str,
        last_name: str,
        document_type: str,
        document_number: str,
        birth_date: str = None,
        phone_number: str = None,
        celphone_number: str = None,
        email: str = None,
        observation: str = None,
        customer_id: str = None,
        seller_id: str = None,
        address: Union[Address, dict] = None,
    ):
        self.seller_id = seller_id
        self.customer_id = customer_id
        self.first_name = first_name
        self.last_name = last_name
        self.document_type = document_type
        self.document_number = document_number
        self.birth_date = birth_date
        self.phone_number = phone_number
        self.celphone_number = celphone_number
        self.email = email
        self.observation = observation

        if isinstance(address, dict):
            address = Address(**address)

        self.address = address

    def __str__(self):
        return "{} ({})".format(self.full_name, self.customer_id)

    def __eq__(self, other):
        return (self.seller_id, self.customer_id) == (
            other.seller_id,
            other.customer_id,
        )

    @property
    def full_name(self):
        return "{} {}".format(self.first_name, self.last_name)

    def as_dict(self):
        data = self.__dict__.copy()
        data["address"] = self.address.as_dict()
        return data



from requests.models import Response

from getnet.usecases import errors


class handler_request:
    def __init__(self, client, logger):
        """
        Args:
            client:
        """
        self.client = client
        self.logger = logger

    def __enter__(self):
        if self.client.access_token_expired():
            self.client.auth()

        return self.logger

    def __exit__(self, exc_type, exc_val, exc_tb):
        """
        Args:
            exc_type:
            exc_val:
            exc_tb:
        """
        pass


def handler_request_exception(response: Response):
    """
    Args:
        response (Response):
    """
    status_code = response.status_code
    data = response.json()
    if "details" in data and len(data.get("details")) > 0:
        data = data.get("details")[0]

    kwargs = {
        "error_code": data.get("error_code")
        or data.get("error")
        or str(data.get("status_code")),
        "description": data.get("description_detail")
        or data.get("description")
        or data.get("error_description")
        or data.get("message"),
        "response": response,
    }

    message = "{} {} ({})".format(
        kwargs.get("error_code"),
        kwargs.get("description"),
        response.url,
    )

    if status_code == 400:
        return errors.BadRequest(message, **kwargs)
    elif status_code == 402:
        return errors.BusinessError(message, **kwargs)
    elif status_code == 404:
        return errors.NotFound(message, **kwargs)
    elif status_code == 500:
        return errors.ServerError(message, **kwargs)
    elif status_code == 503:
        return errors.ServiceUnavailable(message, **kwargs)
    elif status_code == 504:
        return errors.GatewayTimeout(message, **kwargs)
    else:
        return errors.RequestError(message, **kwargs)



from getnet.services.customers import Customer, Address


def test_address_conversion(customer_sample: dict):
    customer = Customer(**customer_sample)

    assert isinstance(customer.address, Address)
    assert (
        customer_sample.get("address").get("postal_code")
        == customer.address.postal_code
    )


def test_full_name(customer_sample: dict):
    customer = Customer(**customer_sample)

    sample_full_name = f'{customer_sample["first_name"]} {customer_sample["last_name"]}'

    assert sample_full_name == customer.full_name



from getnet import Environment


def test_base_url():
    assert "https://api-sandbox.getnet.com.br" == Environment.SANDBOX.base_url()
    assert "https://api-homologacao.getnet.com.br" == Environment.HOMOLOG.base_url()
    assert "https://api.getnet.com.br" == Environment.PRODUCTION.base_url()



from datetime import datetime
from typing import Union

from dateutil import parser

from getnet.domain.payments.payment_response import PaymentResponse


class CreditCancelResponse:
    canceled_at: datetime
    message: str

    def __init__(self, canceled_at: datetime, message: str):
        self.canceled_at = (
            canceled_at
            if isinstance(canceled_at, datetime)
            else parser.isoparse(canceled_at)
        )
        self.message = message


class CreditCancelPaymentResponse(PaymentResponse):
    credit_cancel: CreditCancelResponse

    def __init__(self, credit_cancel: Union[CreditCancelResponse, dict], **kwargs):
        super(CreditCancelPaymentResponse, self).__init__(**kwargs)
        self.credit_cancel = (
            credit_cancel
            if isinstance(credit_cancel, CreditCancelResponse) or credit_cancel is None
            else CreditCancelResponse(**credit_cancel)
        )



import pytest

from getnet.errors import BadRequest, NotFound
from getnet.services.cards import Service, Card
from getnet.services.cards.card_response import NewCardResponse
from getnet.services.service import ResponseList


@pytest.mark.vcr
def test_create(client, card_sample: dict):
    card_sample["number_token"] = client.generate_card_token(
        "5155901222280001", "getnet-py"
    )
    card = Service(client).create(Card(**card_sample))

    assert isinstance(card, NewCardResponse)
    assert card_sample["number_token"] == card.number_token.number_token


@pytest.mark.vcr
def test_invalid_create(client, card_sample: dict):
    with pytest.raises(BadRequest) as excinfo:
        card_sample["number_token"] = "123"
        Service(client).create(Card(**card_sample))

    assert "TOKENIZATION-400" == excinfo.value.error_code


@pytest.mark.vcr
def test_get(client, card_sample: dict):
    card_sample["number_token"] = client.generate_card_token(
        "5155901222280001", "getnet-py"
    )
    service = Service(client)
    sample_card = service.create(Card(**card_sample))

    card = service.get(sample_card.card_id)

    assert isinstance(card, Card)
    assert sample_card.card_id == card.card_id


@pytest.mark.vcr
def test_invalid_get(client):
    with pytest.raises(NotFound) as excinfo:
        Service(client).get("14a2ce5d-ebc3-49dc-a516-cb5239b02285")

    assert "404" == excinfo.value.error_code


@pytest.mark.vcr
def test_all(client, card_sample: dict):
    cards = Service(client).all(card_sample.get("customer_id"))
    assert isinstance(cards, ResponseList)
    assert cards.page is None
    assert cards.limit is None
    assert cards.total is not None


@pytest.mark.vcr
def test_all_not_found(client):
    with pytest.raises(NotFound) as excinfo:
        Service(client).all("foobar")

    assert "404" == excinfo.value.error_code


@pytest.mark.vcr
def test_delete(client, card_sample: dict):
    card_sample["number_token"] = client.generate_card_token(
        "5155901222280001", "getnet-py"
    )
    service = Service(client)

    created_card = service.create(Card(**card_sample))
    card = service.get(created_card.card_id)

    resp = service.delete(card.card_id)
    assert True == resp


@pytest.mark.vcr
def test_delete_not_found(client):
    with pytest.raises(NotFound) as excinfo:
        cards = Service(client).delete("72402c54-6bd3-4895-a6b4-adfded0c11dc")

    assert "VAULT-404" == excinfo.value.error_code



"""
Implements Card Entity
"""

import re
from typing import Union, Optional

from getnet.domain.token.card_token import CardToken

BRANDS = (
    "mastercard",
    "visa",
    "amex",
    "elo",
    "hipercard",
    "Mastercard",
    "Visa",
    "Amex",
    "Elo",
    "Hipercard",
)
CARDHOLDER_IDENTIFICATION_REGEX = re.compile(r"\A\d+\Z")
VERIFY_CODE = re.compile(r"\A\d{3,4}\Z")


class Card(object):
    """Card represents the Card entity for the Cards operations"""

    customer_id: str
    number_token: CardToken
    brand: Union[str, None] = None
    cardholder_name: str
    expiration_month: str
    expiration_year: str
    cardholder_identification: Union[str, None]
    security_code: Union[str, None]
    verify_card: bool = False
    bin: Union[str, None] = None

    def __init__(
        self,
        customer_id: str,
        number_token: Union[CardToken, str],
        cardholder_name: str,
        expiration_month: int,
        expiration_year: int,
        cardholder_identification: str,
        security_code: str,
        verify_card: Optional[bool] = False,
        brand: Union[str, None] = None,
        bin: Union[str, None] = None,
    ):
        """
        Args:
            customer_id (str): Customer Identify
            number_token (CardToken|str): token of the number of card
            cardholder_name (str): Customer name as in the card
            expiration_month (int): Card expiration month
            expiration_year (int): Card expiration year (2 digits)
            cardholder_identification (str): Card owner identify (CPF, RG, etc.)
            security_code (str): Card security code (CVV or CVC)
            verify_card (bool): Optional. If true execute a transaction to check
                the card (cancel, blocked or with restriction)
            brand (str): Optional. Brand name
            bin (str): Ignore is for internal use only
        """
        if brand is not None and brand not in BRANDS:
            raise TypeError("Brand is invalid")

        if not 1 <= int(expiration_month) <= 12 or not 0 <= int(expiration_year) <= 99:
            raise TypeError("Expiration Month or Year must have 2 characters")

        if len(customer_id) > 100:
            raise TypeError("CustomerID must have bellow 100 characters.")

        if (
            cardholder_identification is not None
            and not CARDHOLDER_IDENTIFICATION_REGEX.match(cardholder_identification)
        ):
            raise TypeError("Cardholder identification invalid")

        if security_code is not None and not VERIFY_CODE.match(security_code):
            raise TypeError("Security code must have 3 or 4 characters")

        self.customer_id = customer_id
        self.number_token = (
            number_token
            if isinstance(number_token, CardToken)
            else CardToken(number_token)
        )
        self.cardholder_name = cardholder_name
        self.expiration_month = str(expiration_month)
        self.expiration_year = str(expiration_year)
        self.cardholder_identification = cardholder_identification
        self.security_code = security_code
        self.verify_card = verify_card
        self.brand = brand
        self.bin = bin

    def _as_dict(self):
        """Format the data as dict to be sent to Getnet"""
        data = self.__dict__.copy()
        data.pop("bin")
        data["number_token"] = self.number_token.number_token
        data["expiration_month"] = str(self.expiration_month).zfill(2)
        data["expiration_year"] = str(self.expiration_year).zfill(2)

        if self.security_code is None:
            data.pop("security_code")

        if self.brand is None:
            data.pop("brand")

        return data



from getnet import Client
from getnet.services.customers import Service, Customer
from getnet.services.service import ResponseList


def test_create(
    client_mock: Client, customer_sample: dict, customer_response_sample: dict
):
    client_mock.post.return_value = customer_response_sample

    service = Service(client_mock)
    customer = service.create(Customer(**customer_sample))

    assert isinstance(customer, Customer)
    assert customer_sample.get("customer_id") == customer.customer_id


def test_all(client_mock: Client, customer_response_sample: dict):
    client_mock.get.return_value = {
        "customers": [
            customer_response_sample,
            customer_response_sample,
            customer_response_sample,
        ],
        "page": 1,
        "limit": 100,
        "total": 3,
    }

    service = Service(client_mock)
    customers = service.all()

    assert isinstance(customers, ResponseList)
    assert 1 == customers.page
    assert 3 == customers.total
    assert customer_response_sample.get("customer_id") == customers[0].customer_id


def test_get(client_mock: Client, customer_response_sample: dict):
    client_mock.get.return_value = customer_response_sample

    service = Service(client_mock)
    customer = service.get(customer_response_sample.get("customer_id"))

    assert isinstance(customer, Customer)
    assert customer_response_sample.get("customer_id") == customer.customer_id
    client_mock.get.assert_called_once_with(
        "/v1/customers/{}".format(customer_response_sample.get("customer_id"))
    )



from getnet.domain.customers.customer import Customer


class CustomerResponse(Customer):
    status: str

    def __init__(self, status: str, **kwargs):
        self.status = status
        super(CustomerResponse, self).__init__(**kwargs)



"""Implement Token Service"""

from getnet.domain.services import Service as BaseService
from getnet.domain.token.card_number import CardNumber
from getnet.domain.token.card_token import CardToken
from getnet.infra.dtos.card_token import CardTokenResponse
from pydantic import ValidationError

class Service(BaseService):
    """Represents the token service operations"""

    path = "/v1/tokens/card"

    def generate(self, card: CardNumber):
        """Generate an token for the card data

        Args:
            card (CardNumber):
        """


        self._client.request.headers = (
            {
                "Accept":"application/json, text/plain, */*",
                "Authorization": "Bearer {}".format(self._client.access_token),
                "Content-Type":"application/json"
            }
        )  
        
        response = self._post(self.path, json=card.as_dict())

        try:
            token_data = CardTokenResponse(**response)
        except ValidationError as e:
            raise e.errors()

        return CardToken(token_data.number_token)



from typing import Optional, List
from pydantic import BaseModel, ValidationError, validator


class CardBin(BaseModel):
  country: Optional[str]
  country_code: Optional[str]
  brand_code: Optional[str]
  product: Optional[str]
  brazilian_issued: Optional[str]
  bin: Optional[int]
  country_code_iso: Optional[str]
  type: Optional[str]
  brand: Optional[str]
  issuer: Optional[str]


class CardBinResponse(BaseModel):
    results: List[CardBin]
    status: str = "OK"



import uuid

import pytest

from getnet.services.token.card_token import CardToken


@pytest.fixture
def card_sample():
    return {
        "number_token": CardToken("123"),
        "brand": "visa",
        "cardholder_name": "John Doe",
        "cardholder_identification": "5155901222280001",
        "security_code": "123",
        "expiration_month": "02",
        "expiration_year": "25",
        "customer_id": "johndoe",
        "verify_card": False,
    }.copy()


@pytest.fixture
def card_response_sample():
    return {
        "card_id": uuid.UUID("e8ad2ae4-9e3e-4532-998f-1a5a11e56e58"),
        "number_token": CardToken("123"),
        "brand": "visa",
        "cardholder_name": "John Doe",
        "cardholder_identification": "5155901222280001",
        "security_code": "123",
        "expiration_month": "02",
        "expiration_year": "25",
        "customer_id": "johndoe",
        "verify_card": True,
        "last_four_digits": "1212",
        "used_at": "2017-04-19T16:30:30.003Z",
        "created_at": "2017-04-19T16:30:30.003Z",
        "updated_at": "2017-04-19T16:30:30.003Z",
        "status": "active",
        "bin": "123",
    }.copy()



from datetime import datetime
from typing import Union

from dateutil import parser

from getnet.domain.payments.payment_response import PaymentResponse


class CreditCaptureResponse:
    confirm_date: datetime
    message: str

    def __init__(self, confirm_date: datetime, message: str):
        self.confirm_date = (
            confirm_date
            if isinstance(confirm_date, datetime)
            else parser.isoparse(confirm_date)
        )
        self.message = message


class CreditCapturePaymentResponse(PaymentResponse):
    credit_confirm: CreditCaptureResponse

    def __init__(self, credit_confirm: Union[CreditCaptureResponse, dict], **kwargs):
        super(CreditCapturePaymentResponse, self).__init__(**kwargs)
        self.credit_confirm = (
            credit_confirm
            if isinstance(credit_confirm, CreditCaptureResponse) or credit_confirm is None
            else CreditCaptureResponse(**credit_confirm)
        )



from typing import Union, Optional

from getnet.domain.payments.credit.card import Card

TRANSACTION_FULL = "FULL"
TRANSACTION_INSTALL = "INSTALL_NO_INTEREST"
TRANSACTION_INSTALL_WITH_INTEREST = "INSTALL_WITH_INTEREST"

VALID_TRANSACTION_TYPES = (
    TRANSACTION_FULL,
    TRANSACTION_INSTALL,
    TRANSACTION_INSTALL_WITH_INTEREST,
)


class Credit:
    card: Card
    delayed: bool
    authenticated: bool
    pre_authorization: bool
    save_card_data: bool
    transaction_type: str
    number_installments: int
    soft_descriptor: Optional[Union[str, None]] = None
    dynamic_mcc: Optional[Union[int, None]] = None

    def __init__(
        self,
        card: Union[Card, dict],
        transaction_type: str = TRANSACTION_FULL,
        number_installments: int = 1,
        delayed: bool = False,
        authenticated: bool = False,
        pre_authorization: bool = False,
        save_card_data: bool = False,
        soft_descriptor: str = None,
        dynamic_mcc: int = None,
    ):
        if transaction_type not in VALID_TRANSACTION_TYPES:
            raise TypeError("transaction_type is invalid")

        if soft_descriptor is not None and len(soft_descriptor) > 22:
            raise TypeError("soft_descriptor is too long (max: 23 characters)")

        self.card = card if isinstance(card, Card) or card is None else Card(**card)
        self.delayed = delayed
        self.authenticated = authenticated
        self.pre_authorization = pre_authorization
        self.save_card_data = save_card_data
        self.transaction_type = transaction_type
        self.number_installments = number_installments
        self.dynamic_mcc = dynamic_mcc
        self.soft_descriptor = soft_descriptor

    def as_dict(self):
        data = self.__dict__.copy()
        data["card"] = self.card._as_dict()

        if self.dynamic_mcc is None:
            data.pop("dynamic_mcc")

        if self.soft_descriptor is None:
            data.pop("soft_descriptor")

        return data



"""Implements the CardNumber entity"""
import re
from typing import Optional, Union

from getnet.infra.dtos.authentication import AuthenticationResponse
from getnet.utils import handler_request, handler_request_exception
from datetime import datetime, timedelta
from pydantic import ValidationError


class Authentication:
    client_id: Optional[Union[str, None]] = None
    client_secret: Optional[Union[str, None]] = None
    access_token: Optional[Union[str, None]] = None
    access_token_expires: Optional[Union[int, None]] = None


    def __init__(self, client_id: str, client_secret: str):
        self.client_id = client_id
        self.client_secret = client_secret

    
    def access_token_expired(self) -> bool:
        """Returns true if not have an token or is expired

        Returns:
            bool
        """
        return (
            self.access_token is None
            or self.access_token_expires < datetime.timestamp(datetime.now())
        )


    def auth(self) -> None:
        if self.access_token_expired():
            path = "/auth/oauth/v2/token"
            data = {"scope": "oob", "grant_type": "client_credentials"}

            response = self.request.post(
                self.base_url + path,
                data=data,
                auth=(self.client_id, self.client_secret),
            )

            if not response.ok:
                raise handler_request_exception(response)

            try:
                response_data = AuthenticationResponse(**response.json())
            except ValidationError as e:
                raise e.errors()

            self.access_token = response_data.access_token
            self.access_token_expires = int(
                datetime.timestamp(
                    datetime.now() + timedelta(seconds=response_data.expires_in)
                )
            )
            self.request.headers.update(
                {"Authorization": "Bearer {}".format(self.access_token)}
            )
            


import re
from getnet.infra.dtos.customer import CustomerSchema
from pydantic import ValidationError
from getnet.usecases.errors import RequestError
from getnet.domain.customers.customer import Customer
from getnet.domain.customers.customer_response import CustomerResponse
from getnet.domain.services import Service as BaseService, ResponseList


DOCUMENT_TYPES = ("CPF", "CNPJ")
DOCUMENT_NUMBER_REGEX = re.compile(r"\A\d{11,15}\Z")


class Service(BaseService):
    path = "/v1/customers/{customer_id}"

    def create(self, customer: Customer, return_if_exists: bool = True) -> Customer:
        customer.seller_id = self._client.seller_id
        
        try:
            response = self._post(self._format_url(), json=customer.as_dict())
            try:
                customer_schema = CustomerSchema(**response)
            except ValidationError as e:
                raise e.errors()
            return CustomerResponse(**customer_schema.dict())
        except RequestError as err:
            if return_if_exists and err.error_code == "400":
                return self.get(customer.customer_id)
            else:
                raise

    def all(
        self,
        page: int = 1,
        limit: int = 100,
        customer_id: str = None,
        document_number: str = None,
        first_name: str = None,
        last_name: str = None,
        sort: str = "last_name",
        sort_type: str = "asc",
    ) -> ResponseList:
        if page <= 0:
            raise TypeError("page must be greater then 0")

        if not sort_type in ("asc", "desc"):
            raise AttributeError("sort_type invalid. Choices: asc, desc")

        params = {
            "page": page,
            "limit": limit,
            "customer_id": customer_id,
            "document_number": document_number,
            "first_name": first_name,
            "last_name": last_name,
            "sort": sort,
            "sort_type": "asc",
        }

        response = self._get(self._format_url(), params=params)

        values = [
            CustomerResponse(**customer) for customer in response.get("customers")
        ]

        return ResponseList(
            values, response.get("page"), response.get("limit"), response.get("total")
        )

    def get(self, customer_id: str):
        response = self._get(self._format_url(customer_id=customer_id))
        try:
            customer_schema = CustomerSchema(**response)
        except ValidationError as e:
            raise e.errors()
        return CustomerResponse(**customer_schema.dict())


import pytest

from getnet.services.token import Service, CardNumber
from getnet.services.token.card_token import CardToken


@pytest.mark.vcr()
def test_generate(client):
    service = Service(client)
    number_token = service.generate(CardNumber("5155901222280001", "customer_21081826"))

    assert isinstance(number_token, CardToken)
    assert number_token.number_token is not None



"""
GetNet-Py
=========

This package implements an API client to Santander Getnet.
"""
import logging

from .usecases.client import Client
from .usecases.environment import Environment

__version__ = "1.7"

import requests

if requests.__version__ < "2.0.0":
    msg = (
        "You are using requests version %s, which is older than "
        "getnet-py expects, please upgrade to 2.0.0 or later."
    )
    raise Warning(msg % requests.__version__)

logging.getLogger(__name__).addHandler(logging.NullHandler())



"""This module implement API environment enum"""
from enum import Enum, unique

API_URLS = {
    0: "https://api-sandbox.getnet.com.br",
    1: "https://api-homologacao.getnet.com.br",
    2: "https://api.getnet.com.br",
}


@unique
class Environment(Enum):
    """Environment represents the API envinments specs"""

    SANDBOX = 0
    HOMOLOG = 1
    PRODUCTION = 2

    def base_url(self) -> str:
        return API_URLS[self.value]



import pytest

from getnet import Client
from getnet.errors import BadRequest, NotFound
from getnet.services.customers import Customer, Service
from getnet.services.service import ResponseList


@pytest.mark.vcr
def test_create(client: Client, customer_sample: dict):
    customer_sample["document_number"] = "01234567888"

    customer = Service(client).create(Customer(**customer_sample))

    assert isinstance(customer, Customer)
    assert customer_sample.get("customer_id"), customer.customer_id


@pytest.mark.vcr
def test_invalid_create(client: Client, customer_sample: dict):
    with pytest.raises(BadRequest) as excinfo:
        Service(client).create(Customer(**customer_sample), False)

    assert "400" == excinfo.value.error_code


@pytest.mark.vcr
def test_create_return_if_already_exists(client: Client, customer_sample: dict):
    customer = Service(client).create(Customer(**customer_sample), True)

    assert isinstance(customer, Customer)
    assert customer_sample.get("customer_id"), customer.customer_id


@pytest.mark.vcr
def test_get(client: Client, customer_sample: dict):
    customer_sample["customer_id"] = "test_integration_get"
    customer_sample["document_number"] = "01234567811"
    service = Service(client)
    created_customer = service.create(Customer(**customer_sample))

    customer = service.get(created_customer.customer_id)

    assert isinstance(customer, Customer)
    assert created_customer == customer
    assert created_customer.customer_id == customer.customer_id


@pytest.mark.vcr
def test_invalid_get(client):
    with pytest.raises(NotFound) as excinfo:
        Service(client).get("14a2ce5d-ebc3-49dc-a516-cb5239b02285")

    assert "404" == excinfo.value.error_code


@pytest.mark.vcr
def test_all(client):
    customers = Service(client).all()
    assert isinstance(customers, ResponseList)
    assert 1 == customers.page
    assert 100 == customers.limit
    assert customers.total is not None


@pytest.mark.vcr
def test_all_not_found(client):
    cards = Service(client).all(document_number="01234567855")
    assert 0 == cards.total



from getnet.domain.cards import Card as BaseCard


class Card(BaseCard):
    def __init__(self, **kwargs):
        kwargs.update({"customer_id": "", "cardholder_identification": None})

        super(Card, self).__init__(**kwargs)

    def _as_dict(self):
        data = super(Card, self)._as_dict()
        data.pop("customer_id")
        data.pop("verify_card")
        data.pop("cardholder_identification")

        return data



class Address:
    street: str
    number: str
    complement: str
    district: str
    city: str
    state: str
    country: str
    postal_code: str

    def __init__(
        self,
        street: str,
        number: str,
        complement: str,
        district: str,
        city: str,
        state: str,
        country: str,
        postal_code: str,
    ):
        self.street = street
        self.number = number
        self.complement = complement
        self.district = district
        self.city = city
        self.state = state
        self.country = country
        self.postal_code = postal_code

    def as_dict(self):
        return self.__dict__.copy()



from datetime import datetime
from typing import Union

from dateutil import parser

from getnet.domain.payments.credit.credit import Credit
from getnet.domain.payments.payment_response import PaymentResponse


class CreditResponse(Credit):
    authorization_code: str
    authorized_at: datetime
    reason_code: int
    reason_message: str
    acquirer: str
    acquirer_transaction_id: str
    terminal_nsu: str
    transaction_id: str
    brand: str
    # first_installment_amount: str
    # other_installment_amount: str
    # total_installment_amount: str

    def __init__(
        self,
        authorization_code: str,
        authorized_at: Union[datetime, str],
        reason_code: int,
        reason_message: str,
        acquirer: str,
        acquirer_transaction_id: str,
        terminal_nsu: str,
        transaction_id: str,
        brand: str,
        # first_installment_amount: str,
        # other_installment_amount: str,
        # total_installment_amount: str,
        **kwargs,
    ):
        self.authorization_code = authorization_code
        self.authorized_at = (
            authorized_at
            if isinstance(authorized_at, datetime)
            else parser.isoparse(authorized_at)
        )
        self.reason_code = reason_code
        self.reason_message = reason_message
        self.acquirer = acquirer
        self.acquirer_transaction_id = acquirer_transaction_id
        self.terminal_nsu = terminal_nsu
        self.transaction_id = transaction_id
        self.brand = brand
        # self.first_installment_amount = first_installment_amount
        # self.other_installment_amount = other_installment_amount
        # self.total_installment_amount = total_installment_amount
        kwargs.update({"card": None})
        super(CreditResponse, self).__init__(**kwargs)


class CreditPaymentResponse(PaymentResponse):
    credit: CreditResponse

    def __init__(self, credit: Union[CreditResponse, dict], **kwargs):
        super(CreditPaymentResponse, self).__init__(**kwargs)
        self.credit = (
            credit
            if isinstance(credit, CreditResponse) or credit is None
            else CreditResponse(**credit)
        )



import logging
from datetime import datetime, timedelta
from typing import Union, Optional

import requests
from getnet.domain.authentication import Authentication

from getnet.usecases.environment import Environment
from getnet.domain import payments
from getnet.domain.card_bin import CardBinInfo
from getnet.domain.verification import CardVerificationService, CardVerification
from getnet.domain.token import Service as TokenService
from getnet.domain.customers import Service as CustomersService
from getnet.domain.payments.credit import card
from getnet.domain.token.card_token import CardToken
from getnet.domain.token.card_number import CardNumber
from getnet.domain.customers import Customer, Address
from getnet.utils import handler_request, handler_request_exception

__all__ = ["Client"]

LOGGER = logging.getLogger(__name__)


class Client(object):
    """Return an SDK client"""

    request: requests.Session
    seller_id: Optional[Union[str, None]] = None
    client_id: Optional[Union[str, None]] = None
    client_secret: Optional[Union[str, None]] = None
    environment: Environment = Environment.SANDBOX
    access_token: Optional[Union[str, None]] = None
    access_token_expires: Optional[Union[int, None]] = None
    

    def __init__(
        self,
        seller_id: str,
        client_id: str,
        client_secret: str,
        environment: Environment = Environment.SANDBOX,
    ):
        """
        Args:
            seller_id (str):
            client_id (str):
            client_secret (str):
            environment (Environment):
        """
        self.seller_id = seller_id
        self.client_id = client_id
        self.client_secret = client_secret

        if not isinstance(environment, Environment):
            raise AttributeError("Invalid environment")

        self.environment = environment
        self.base_url = environment.base_url()

        self._setup_client()

    def _setup_client(self) -> None:
        self.request = requests.Session()
        self.request.headers.update(
            {"user-agent": "getnet-py/1.1", "seller_id": self.seller_id}
        )

    def _handler_request(self):
        return handler_request(self, LOGGER)

    def access_token_expired(self) -> Authentication.access_token_expired:
        return Authentication.access_token_expired(self)
    
    def auth(self) -> Authentication.auth:
        return Authentication.auth(self)

    
    def get(self, path, **kwargs) -> dict:
        """
        Args:
            path:
            **kwargs:
        """
        with handler_request(self, LOGGER):
            url = self.base_url + path
            response = self.request.get(url, **kwargs)
            if not response.ok:
                raise handler_request_exception(response)
            return response.json()

    def post(self, path: str, **kwargs) -> dict:
        """
        Args:
            path (str):
            **kwargs:
        """
        with handler_request(self, LOGGER):
            url = self.base_url + path
            response = self.request.post(url, **kwargs)
            if not response.ok:
                raise handler_request_exception(response)
            return response.json()

    def patch(self, path: str, **kwargs) -> dict:
        """
        Args:
            path (str):
            **kwargs:
        """
        with handler_request(self, LOGGER):
            url = self.base_url + path
            response = self.request.patch(url, **kwargs)
            if not response.ok:
                raise handler_request_exception(response)
            return response.json()

    def delete(self, path: str, **kwargs) -> Union[bool, dict]:
        """
        Args:
            path (str):
            **kwargs:
        """
        with handler_request(self, LOGGER):
            url = self.base_url + path
            response = self.request.delete(url, **kwargs)
            if not response.ok:
                raise handler_request_exception(response)

            return True

    def generate_card_token(self, card_number: str, customer_id: str) -> CardToken:
        """Shortcut to card token generation

        Args:
            card_number (str): credit card number
            customer_id (str): Customer identify

        Raises:
            * AttributeError, RequestError
        """
        return TokenService(self).generate(CardNumber(card_number, customer_id))


    def card_verified(self, number_token, expiration_month, expiration_year,  cardholder_name, brand=None, security_code=None) -> bool:
        return CardVerificationService(self).verification(CardVerification(number_token, expiration_month, expiration_year, cardholder_name, brand, security_code))

    def card_bin(self, card_bin: str) -> dict:
        return CardBinInfo(self).binlookup(card_bin)

    def order(self, order_id: str, sales_tax: int = None, product_type: str = None) -> payments.Order:
        return payments.Order(order_id, sales_tax, product_type)
    
    def customer_service(self) -> CustomersService:
        """Return a instance of token service"""
        return CustomersService(self)

    def customer(self, customer_data) -> Customer:
        return Customer(**customer_data)

    def payment_customer(self, customer) -> payments.Customer:
        return payments.Customer(customer)
    
    # Payments  Credit
    def payment_credit_service(self) -> payments.credit.Service:
        """Return a instance of token service"""
        return payments.credit.Service(self)

    def credit_card(self,number_token, cardholder_name, security_code, brand, expiration_month, expiration_year) -> card.Card:
        return card.Card(number_token=number_token, cardholder_name=cardholder_name, security_code=security_code, brand=brand, expiration_month=expiration_month, expiration_year=expiration_year)

    def create_credit_transaction(self, amount, delayed, pre_authorization, save_card_data, transaction_type, number_installments, order, customer, card, shipping_address, currency="BRL") -> payments.credit.Service:
        credit = {
            "delayed": delayed,
            "pre_authorization": pre_authorization,
            "save_card_data": save_card_data,
            "transaction_type": transaction_type,
            "number_installments": number_installments,
            "card": card._as_dict(),
        }

        return self.payment_credit_service().create(amount, currency, order, credit, customer, shipping_address)

    def cancel_credit_transaction(self, payment_id):
        return self.payment_credit_service().cancel(payment_id)

    def capture_credit_transaction(self, payment_id: str, amount: str):
        return self.payment_credit_service().capture(payment_id, amount)

    def adjust_credit_transaction(self, payment_id: str, amount: str, currency="BRL"):
        return self.payment_credit_service().adjust(payment_id, amount, currency)


import pytest

from getnet.services.cards.card import Card
from getnet.services.token.card_token import CardToken


def test_invalid_expiration_month(card_sample: dict):
    with pytest.raises(TypeError):
        card_sample["expiration_month"] = 13
        Card(**card_sample)


def test_invalid_expiration_year(card_sample: dict):
    with pytest.raises(TypeError):
        card_sample["expiration_year"] = 100
        Card(**card_sample)


def test_invalid_customer_id(card_sample: dict):
    with pytest.raises(TypeError):
        card_sample["customer_id"] = "1" * 101
        Card(**card_sample)


def test_invalid_security_code2(card_sample: dict):
    with pytest.raises(TypeError):
        card_sample["security_code"] = "12"
        Card(**card_sample)


def test_invalid_security_code5(card_sample: dict):
    with pytest.raises(TypeError):
        card_sample["security_code"] = "12345"
        Card(**card_sample)


def test_number_token_as_str(card_sample: dict):
    card_sample["number_token"] = "12345"
    card = Card(**card_sample)

    assert isinstance(card.number_token, CardToken)
    assert "12345" == card.number_token.number_token


def test_invalid_brand(card_sample: dict):
    with pytest.raises(TypeError):
        card_sample["brand"] = "12345"
        Card(**card_sample)


def test_as_dict(card_sample: dict):
    card = Card(**card_sample)
    assert card_sample == card._as_dict()



import unittest
from unittest.mock import patch

import pytest
from pytest_mock import MockFixture

from getnet.services.token import Service, CardNumber
from getnet.services.token.card_token import CardToken


@pytest.fixture
def card_number():
    return CardNumber("5155901222280001", "customer_21081826")


def test_token_generate(client_mock, card_number):
    client_mock.post.return_value = {"number_token": "123456789"}

    service = Service(client_mock)
    token = service.generate(card_number)

    assert isinstance(token, CardToken)
    assert "123456789" == token.number_token



"""Implements the CardNumber entity"""
import re


class CardVerification:
    """Implements the CardNumber data required to generate the tokenized
    version
    """

    number_token: str
    brand: str
    cardholder_name: str
    expiration_month: str
    expiration_year: str
    security_code: str


    def __init__(self, number_token: str, expiration_month: str, expiration_year: str, cardholder_name: str, brand: str = None, security_code: str = None):
        """
        Args:
            card_number (str): An valid card number
            customer_id (str): Customer identify
        """
        if not number_token:
            raise AttributeError(
                "Card Token is invalid or empty!"
            )

        self.number_token = number_token
        self.expiration_month = expiration_month
        self.expiration_year = expiration_year
        
        if brand:
            self.brand = brand
        if cardholder_name:
            self.cardholder_name = cardholder_name
        if security_code:
            self.security_code = security_code


    def as_dict(self) -> dict:
        """Format the data as dict to be sent to Getnet"""
        return self.__dict__.copy()



"""Implement Token Service"""

from getnet.domain.services import Service as BaseService
from getnet.domain.verification.card_verification import CardVerification
from getnet.domain.verification.card_verified import CardVerified
from getnet.infra.dtos.card_verification import CardVerificationResponse
from pydantic import ValidationError


class CardVerificationService(BaseService):
    """Represents the token service operations"""

    path = "/v1/cards/verification"

    def verification(self, card: CardVerification):
        """Generate an token for the card data

        Args:
            card (Card:
        """

        self._client.request.headers = (
            {
                "Accept":"application/json, text/plain, */*",
                "Authorization": "Bearer {}".format(self._client.access_token),
                "Content-Type":"application/json"
            }
        )
        
        response = self._post(self.path, json=card.as_dict())
        
        try:
            card_verification_data = CardVerificationResponse(**response)
        except ValidationError as e:
            raise e.errors()

        return CardVerified(card_verification_data.status)



from datetime import datetime
from typing import Union

from dateutil import parser

from getnet.domain.payments.payment_response import PaymentResponse


class CreditAdjustPaymentResponse(PaymentResponse):
    authorization_code: str
    authorized_at: datetime
    reason_code: int
    reason_message: str
    acquirer: str
    soft_descriptor: str
    terminal_nsu: str
    acquirer_transaction_id: str
    adjustment_acquirer_transaction_id: str
    
    def __init__(
        self,
        authorization_code: str,
        authorized_at: Union[datetime, str],
        reason_code: int,
        reason_message: str,
        acquirer: str,
        soft_descriptor: str,
        terminal_nsu: str,
        acquirer_transaction_id: str,
        adjustment_acquirer_transaction_id: str,
        **kwargs,
    ):
        self.authorization_code = authorization_code
        self.authorized_at = (
            authorized_at
            if isinstance(authorized_at, datetime)
            else parser.isoparse(authorized_at)
        )
        self.reason_code = reason_code
        self.reason_message = reason_message
        self.acquirer = acquirer
        self.acquirer_transaction_id = acquirer_transaction_id
        self.terminal_nsu = terminal_nsu
        self.soft_descriptor = soft_descriptor
        self.adjustment_acquirer_transaction_id = adjustment_acquirer_transaction_id

        super(CreditAdjustPaymentResponse, self).__init__(**kwargs)


from datetime import timedelta, datetime

import pytest
from pytest_mock import MockerFixture

from getnet import Client
from getnet.usecases.errors import RequestError
from getnet.services import cards, customers
from getnet.domain.token.card_token import CardToken
from getnet.domain import token
import os

seller_id = os.getenv('SELLER_ID')
client_id = os.getenv('CLIENT_ID')
client_secret = os.getenv('CLIENT_SECRET')

@pytest.fixture
def client():
    return Client(
        seller_id,
        client_id,
        client_secret,
    )


class TestClientAuth:
    def test_invalid_data(self, client, mocker: MockerFixture) -> None:
        session_get_mock = mocker.patch(
            "requests.Session.get", return_value=mocker.MagicMock()
        )
        session_get_mock.ok.return_value = False

        with pytest.raises(RequestError):
            client.auth()

    def test_missing_access_token(self, client, mocker: MockerFixture) -> None:
        mocker.patch("getnet.Client.auth", return_value=True)
        session_get_mock = mocker.patch(
            "requests.Session.get", return_value=mocker.MagicMock()
        )
        session_get_mock.ok.return_value = True

        access_token_expired = mocker.spy(client, "access_token_expired")
        client.access_token = None

        client.get("/test")
        access_token_expired.assert_called_once()
        client.auth.assert_called_once()

    def test_expired_access_token(self, client, mocker: MockerFixture) -> None:
        mocker.patch("getnet.Client.auth", return_value=True)
        session_get_mock = mocker.patch(
            "requests.Session.get", return_value=mocker.MagicMock()
        )
        session_get_mock.ok.return_value = True

        access_token_expired = mocker.spy(client, "access_token_expired")
        client.access_token = "test"
        client.access_token_expires = int(
            datetime.timestamp(datetime.now() + timedelta(seconds=-3600))
        )

        client.get("/test")
        access_token_expired.assert_called_once()
        client.auth.assert_called_once()


def test_token_service(client, mocker: MockerFixture) -> None:
    mocker.patch("getnet.Client.auth", return_value=True)

    assert isinstance(client.token_service(), token.Service)


def test_generate_token_card_shortcut(client, mocker: MockerFixture):
    mocker.patch("getnet.Client.auth", return_value=True)
    token_service_mock = mocker.patch.object(token.Service, "generate")
    token_service_mock.return_value = CardToken("123")

    response = client.generate_card_token("5155901222280001", "customer_21081826")

    assert response.number_token == "123"
    token_service_mock.assert_called_once()



import re

import setuptools

# Get the version
version_regex = r'__version__ = ["\']([^"\']*)["\']'
with open("getnet/__init__.py", "r") as f:
    text = f.read()
    match = re.search(version_regex, text)

    if match:
        VERSION = match.group(1)
    else:
        raise RuntimeError("No version number found!")

print("Version: {}".format(VERSION))

APP_NAME = "getnet-python"

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name=APP_NAME,
    version=VERSION,
    description="A Getnet SDK",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Fabio Vitor",
    author_email="fabvitor2010@gmail.com",
    url="https://github.com/FVitor7/getnet-python",
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
    install_requires=["requests>=2.0.0", "requests-oauthlib>=1.2.0", "python-dateutil==2.8.1"],
    classifiers=[
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
    zip_safe=False,
)



class Customer:
    customer_id: str
    first_name: str
    last_name: str
    document_type: str
    document_number: str
    email: str

    billing_address: dict

    def __init__(self, customer: str):
        if len(customer.customer_id) > 100:
            raise TypeError("The customer_id must have bellow of 100 characters")

        self.customer_id = customer.customer_id
        self.first_name = customer.first_name
        self.last_name = customer.last_name
        self.document_type = customer.document_type
        self.document_number = customer.document_number
        self.email = customer.email
        self.billing_address = customer.address.as_dict()

    def as_dict(self):
        return self.__dict__.copy()



PRODUCT_TYPES = (
    "cash_carry",
    "digital_content",
    "digital_goods",
    "digital_physical",
    "gift_card",
    "physical_goods",
    "renew_subs",
    "shareware",
    "service",
)


class Order:
    order_id: str
    sales_tax: int
    product_type: str

    def __init__(self, order_id: str, sales_tax: int, product_type: str):
        if len(order_id) > 36:
            raise TypeError("The order_id must have bellow of 32 characters")

        if product_type and product_type not in PRODUCT_TYPES:
            raise TypeError("The product_type is invalid")

        self.order_id = order_id
        if sales_tax:
            self.sales_tax = sales_tax
        if product_type:
            self.product_type = product_type

    def as_dict(self):
        return self.__dict__.copy()



"""Implements the CardNumber entity"""
import re


class CardNumber:
    """Implements the CardNumber data required to generate the tokenized
    version
    """

    card_number: str
    customer_id: str

    card_number_regex = re.compile(r"\A\d{13,19}\Z")

    def __init__(self, card_number: str, customer_id: str):
        """
        Args:
            card_number (str): An valid card number
            customer_id (str): Customer identify
        """
        if not self.card_number_regex.match(card_number):
            raise AttributeError(
                "Card Number is invalid, must contain only numbers and between 13 and 19 chars"
            )

        if len(customer_id) > 100:
            raise AttributeError("CustomerID must have bellow 100 characters.")

        self.card_number = card_number
        self.customer_id = customer_id

    def __str__(self):
        return self.card_number

    def __eq__(self, other):
        return (
            self.card_number == other.card_number
            and self.customer_id == other.customer_id
        )

    def as_dict(self) -> dict:
        """Format the data as dict to be sent to Getnet"""
        return self.__dict__.copy()



"""
Implement Cards (cofre) Service
"""
import logging
import uuid
from typing import List, Union

from getnet.domain.cards.card import Card
from getnet.domain.cards.card_response import CardResponse, NewCardResponse
from getnet.domain.cards.status import Status
from getnet.domain.services import Service as BaseService, ResponseList
from getnet.domain.token.card_token import CardToken

LOGGER = logging.getLogger(__name__)


class Service(BaseService):
    """Service implements the Card service operations"""

    path = "/v1/cards/{card_id}"

    def verify(self, card: Card) -> bool:
        """Checks if the card is valid

        Args:
            card (Card): Customer card data
        """
        response = self._post(
            self._format_url(card_id="verification"), json=card._as_dict()
        )
        return response.get("status") == "VERIFIED"

    def create(self, card: Card) -> NewCardResponse:
        """Store the card in the safe

        Args:
            card (Card): Customer card data
        """
        response = self._post(self._format_url(), json=card._as_dict())
        return NewCardResponse(**response)

    def all(self, customer_id: str, status: Status = Status.ALL) -> List[CardResponse]:
        """Return an list of cards saved on safe for customer_id

        Args:
            customer_id (str): customer identify
            status: Filter of status of returned cards. Default to "all"
        """
        if status and not status in Status:
            raise AttributeError("Invalid status.")

        params = {"status": status}
        if customer_id is not None:
            params.update({"customer_id": customer_id})

        response = self._get(self._format_url(), params=params)

        cards = []
        for card in response.get("cards"):
            cards.append(CardResponse(**card))

        return ResponseList(
            cards,
            response.get("page", None),
            response.get("limit", None),
            response.get("total", len(cards)),
        )

    def get(self, card_id: Union[CardToken, uuid.UUID, str]) -> CardResponse:
        """Return the data of an saved card on the safe

        Args:
            card_id: Unique card identify on Getnet Plataform
        """
        response = self._get(self._format_url(card_id=str(card_id)))
        return CardResponse(**response)

    def delete(self, card_id: Union[CardToken, uuid.UUID, str]) -> bool:
        """Remove an card saved on safe

        Args:
            card_id: Unique card identify on Getnet Plataform
        """
        self._delete(self._format_url(card_id=str(card_id)))
        return True



from getnet import Client
from getnet.services.cards import Service, Card
from getnet.services.cards.card_response import NewCardResponse
from getnet.services.service import ResponseList


def test_create(client_mock: Client, card_sample: dict):
    client_mock.post.return_value = {
        "card_id": "e8ad2ae4-9e3e-4532-998f-1a5a11e56e58",
        "number_token": "123",
    }

    service = Service(client_mock)
    card = service.create(Card(**card_sample))

    assert isinstance(card, NewCardResponse)
    assert "e8ad2ae4-9e3e-4532-998f-1a5a11e56e58" == str(card.card_id)
    assert "123" == card.number_token


def test_all(client_mock: Client, card_response_sample: dict):
    client_mock.get.return_value = {
        "cards": [card_response_sample, card_response_sample, card_response_sample]
    }

    service = Service(client_mock)
    cards = service.all("client_id")

    assert isinstance(cards, ResponseList)
    assert cards.page is None
    assert 3 == cards.total
    assert card_response_sample.get("card_id") == cards[0].card_id


def test_delete(client_mock: Client):
    client_mock.delete.return_value = True

    service = Service(client_mock)
    card = service.delete(card_id="123")

    assert card == True
    client_mock.delete.assert_called_once_with("/v1/cards/123")


def test_verify(client_mock: Client, card_sample: dict):
    client_mock.post.return_value = {"status": "VERIFIED"}

    service = Service(client_mock)
    response = service.verify(Card(**card_sample))

    assert isinstance(response, bool)
    assert True == response



from typing import Union
from uuid import UUID

from getnet.domain.payments import Customer
from getnet.domain.payments.credit.credit import Credit
from getnet.domain.payments.credit.credit_adjust import CreditAdjustPaymentResponse
from getnet.domain.payments.credit.credit_cancel import CreditCancelPaymentResponse
from getnet.domain.payments.credit.credit_capture import CreditCapturePaymentResponse
from getnet.domain.payments.credit.credit_response import CreditPaymentResponse
from getnet.domain.payments.order import Order
from getnet.domain.services import Service as BaseService
#from getnet.services.utils import Device


class Service(BaseService):
    path = "/v1/payments/credit"

    def create(
        self,
        amount: int,
        currency: str,
        order: Order,
        credit: Credit,
        customer: Customer,
        #device: Device = None,
        shipping_address: dict = None,
    ) -> CreditPaymentResponse:

        data = {
            "seller_id": self._client.seller_id,
            "amount": amount,
            "currency": currency,
            "order": order.as_dict(),
            "credit": credit, #.as_dict(),
            "customer": customer.as_dict(),
            "device": {},
            "shippings": [{"address":shipping_address}],
        }

        # if device is not None:
        #     data["device"] = device.as_dict()

        self._client.request.headers = {
            "Accept": "application/json, text/plain, */*",
            "Content-Type": "application/json",
            "Authorization": "Bearer {}".format(self._client.access_token),
            "seller_id": self._client.seller_id
            }
        print(data)
        response = self._post(self._format_url(), json=data)
        print(response)
        
        return CreditPaymentResponse(**response)

    def cancel(self, payment_id: Union[UUID, str]) -> CreditCancelPaymentResponse:
        response = self._post(
            self._format_url(path="/{payment_id}/cancel", payment_id=str(payment_id))
        )
        return CreditCancelPaymentResponse(**response)

    def capture(
        self,
        payment_id: Union[UUID, str], amount
    ) -> CreditCapturePaymentResponse:
      
        self._client.request.headers = {
            "Content-Type": "application/json; charset=utf-8",
            "Authorization": "Bearer {}".format(self._client.access_token),
            }

        data = {
            "amount":amount
        }

        response = self._post(self._format_url(path="/{payment_id}/confirm", payment_id=str(payment_id)), json=data)

        return CreditCapturePaymentResponse(**response)


    def adjust(
        self,
        payment_id: Union[UUID, str], amount: str, currency="BRL"
    ) -> CreditAdjustPaymentResponse:
      
        self._client.request.headers = {
            "Content-Type": "application/json; charset=utf-8",
            "Authorization": "Bearer {}".format(self._client.access_token),
            }

        data = {
            "amount":amount,
            "currency": currency,
        }

        response = self._post(self._format_url(path="/{payment_id}/adjustment", payment_id=str(payment_id)), json=data)
        return CreditAdjustPaymentResponse(**response)


import pytest

from getnet.services.cards.card_response import CardResponse


def test_invalid_without_card_id(card_response_sample: dict):
    with pytest.raises(TypeError):
        card_response_sample.pop("card_id")
        CardResponse(**card_response_sample)


def testAsDict(card_response_sample: dict):
    card = CardResponse(**card_response_sample)
    value = card._as_dict()
    assert "used_at" not in value
    assert "created_at" not in value
    assert "updated_at" not in value



import pytest


@pytest.fixture
def customer_sample():
    return {
        "seller_id": "6eb2412c-165a-41cd-b1d9-76c575d70a28",
        "customer_id": "customer_21081826",
        "first_name": "João",
        "last_name": "da Silva",
        "document_type": "CPF",
        "document_number": "78075751159",
        "birth_date": "1976-02-21",
        "phone_number": "5551999887766",
        "celphone_number": "5551999887766",
        "email": "customer@email.com.br",
        "observation": "O cliente tem interesse no plano x.",
        "address": {
            "street": "Av. Brasil",
            "number": "1000",
            "complement": "Sala 1",
            "district": "São Geraldo",
            "city": "Porto Alegre",
            "state": "RS",
            "country": "Brasil",
            "postal_code": "90230060",
        },
    }.copy()


@pytest.fixture
def customer_response_sample(customer_sample):
    data = customer_sample.copy()
    data["status"] = "active"
    return data.copy()



"""
Base service module
"""

import re


class Service(object):
    """
    Service Abstract an service needs
    """

    _client = None
    path: str = None

    def __init__(self, client: "Client") -> None:
        if not self.path:
            raise NotImplementedError("The classes parameter path must be defined")

        self._client = client

    def _format_url(self, path=None, **kwargs) -> str:
        data = {}

        path = self.path + path if path is not None else self.path

        match = re.search(r"{(\w+)}", path)
        if match:
            data = {}.fromkeys(list(match.groups()), "")

        data.update(**kwargs)

        return path.format(**data).rstrip("/")

    def _get(self, *args, **kwargs):
        return self._client.get(*args, **kwargs)

    def _post(self, *args, **kwargs):
        return self._client.post(*args, **kwargs)

    def _patch(self, *args, **kwargs):
        return self._client.patch(*args, **kwargs)

    def _delete(self, *args, **kwargs):
        return self._client.delete(*args, **kwargs)


class ResponseList(list):
    def __init__(self, seq=(), page=1, limit=100, total=None):
        self.page = page
        self.limit = limit
        self.total = total
        super(ResponseList, self).__init__(seq)



from getnet.usecases.errors.request_error import RequestError

__ALL__ = [
    "BadRequest",
    "NotFound",
    "ServerError",
    "ServiceUnavailable",
    "GatewayTimeout",
]


class BadRequest(RequestError):
    pass


class NotFound(RequestError):
    pass


class ServerError(RequestError):
    pass


class ServiceUnavailable(RequestError):
    pass


class GatewayTimeout(RequestError):
    pass



"""Implements the Card Token entity"""


class CardToken:
    """Represent the card number tokenized version"""

    number_token: str

    def __init__(self, number_token: str):
        """
        Args:
            number_token (str):
        """
        self.number_token = number_token

    def __str__(self):
        return str(self.number_token)

    def __eq__(self, other):
        match = other.number_token if isinstance(other, CardToken) else other
        return self.number_token == match



import ipaddress

import pytest

from getnet.services.utils import Device


class TestDevice:
    def test_invalid_device_id(self):
        with pytest.raises(TypeError):
            Device("127.0.0.1", "A" * 81)

    def test_invalid_ipaddress(self):
        with pytest.raises(ipaddress.AddressValueError):
            Device("127.0.0.300", "ABC")

    def test_get_as_dict(self):
        object = Device("127.0.0.3", "ABC")
        assert {"ip_address": "127.0.0.3", "device_id": "ABC"} == object.as_dict()



from getnet.usecases.errors.request_error import RequestError


class BusinessError(RequestError):
    @property
    def details(self):
        return self.response.json().get("details")[0]

    @property
    def payment_id(self):
        return self.details.get("payment_id")

    @property
    def authorization_code(self):
        return self.details.get("authorization_code")

    @property
    def terminal_nsu(self):
        return self.details.get("terminal_nsu")

    @property
    def acquirer_transaction_id(self):
        return self.details.get("acquirer_transaction_id")

    @property
    def status(self):
        return self.details.get("status")
